/*******************************************************************
*
*  DESCRIPTION: 
*
*  AUTHOR: Amir Barylko & Jorge Beyoglonian 
*
*  EMAIL: mailto://amir@dc.uba.ar
*         mailto://jbeyoglo@dc.uba.ar
*
*  DATE: 27/6/1998
*
*******************************************************************/


/** include files **/
#include <time.h>

/** my include files **/
#include "trafico.h"    // class Trafico
#include "message.h"    // class ExternalMessage, InternalMessage

/** public functions **/

/*******************************************************************
* Function Name: Trafico
* Description: 
********************************************************************/
Trafico::Trafico( const string &name )
: Atomic( name )
, in( this->addInputPort( "in" ) )
, east( this->addOutputPort( "east" ) )
, west( this->addOutputPort( "west" ) )
, north( this->addOutputPort( "north" ) )
, south( this->addOutputPort( "south" ) )
, demoringTime( 0, 0, 1, 0 )
{}


/*******************************************************************
* Function Name: initFunction
* Description: 
* Precondition: El tiempo del proximo evento interno es Infinito
********************************************************************/ 
Model &Trafico::initFunction()
{
	time_t t;
	time (&t);
	#ifdef __unix__
	srandom( t );
	#else
	srand( t );
	#endif
	return *this ;
}


/*******************************************************************
* Function Name: externalFunction
* Description: si estaba en modo activo (ya habia un vehiculo
*              transitando la manzana) ==> chocan y termina!!!
********************************************************************/
Model &Trafico::externalFunction( const ExternalMessage & )
{
	if( this->state() == passive )
		this->holdIn( active, demoringTime );
	else
		this->passivate();

	return *this;
}

/*******************************************************************
* Function Name: internalFunction
* Description: 
********************************************************************/
Model &Trafico::internalFunction( const InternalMessage & )
{
	this->passivate();
	return *this ;
}


/*******************************************************************
* Function Name: outputFunction
* Description: se elige la direccion por la que saldra el vehiculo
********************************************************************/
Model &Trafico::outputFunction( const InternalMessage &msg )
{
	#ifdef __unix__
	switch( random () % 4 )
	#else
	switch( rand () % 4 )
	#endif
	{
		case 0:  this->sendOutput( msg.time(), north, 0 ) ;
			break;
		case 1:  this->sendOutput( msg.time(), east, 0 ) ;
			break;
		case 2:  this->sendOutput( msg.time(), south, 0 ) ;
			break;
		case 3:  this->sendOutput( msg.time(), west, 0 ) ;
			break;
	}

	return *this ;
}



/** private functions **/
